;+
; NAME: 
;       INPAINTING
;
; PURPOSE: 
;       process the inpainting of convergence map from incomplete shear maps
;
; CALLING:
;       inpainting, ggm, niter, DCTBlockSize=256, nscale= 4, out, /expo, /bounded, /sbounded, res 
;
; INPUTS: 
;       gm --- shear structure
;	niter --- nb of iterations
;
; KEYWORDS:
;	DCTBlocksize --- size of blocks in the local DCT
;	nscale --- nb of wavelet scales
;	FirstT --- First Threshold
;	LastT --- Last Threshold
;	expo --- exponential decrease of the threshold
;	bounded --- inside the gaps the maximum and the minimum is bounded
;	sbounded --- inside the gaps the variance is limited
;	
; OUTPUT: 
;       out_data --- inpainted map
;	residual --- residual map			    
;
; HISTORY:
;	Written: S. Pires Feb. 2009
;-
;---------------------------------------------------------
pro inpainting, gm, niter, iter=iter, DCTBlockSize = DCTBlockSize, $
out_data, nscale=nscale, FirstT=FirstT, LastT=LastT, expo=expo,bounded=bounded,$
sbounded=sbounded,residual
g1m = gm.gamma1
g2m = gm.gamma2
print, 'Inpainting with the DCT'
;variables declaration
sz = size(g1m)
n1 = sz[1]
n2 = sz[2]
field = 3.95*float(n1)/1024.
if n1 ne n2 then print, 'image is not squared'
out_data = fltarr(n1, n2)
tempo_data = fltarr(n1, n2)
mask = (g1m ne 0)
ze = where(mask eq 0, c)
nze = where(mask ne 0)
;my_plt, mask
if not keyword_set(nscale) then nscale = fix(alog(n1)/(alog(2.))) - 3.- 2.  
gamma_to_kappa, gm, mm
if keyword_set(bounded) then begin
  BoundMin = min(mm.kappa*(mask))
  BoundMax = max(mm.kappa*(mask))
endif

if not keyword_set(DCTBlockSize) then DCTBlockSize = n1
if not keyword_set(iter) then iter = 0
;First Threshold processing
if not keyword_set(FirstT) then begin
  im_dct, mm.kappa, out, opt='-O -b'+string(DCTBlockSize)
  outn =out
  sz = size(outn)
  ndct = sz[1]
  i = 0
  j = 0
  while i lt ndct do begin
    while j lt ndct do begin
      ;print, i, j
      outn(i, j) = 0
      j = j + DCTBlockSize
    endwhile
    i = i + DCTBlockSize
    j = 0
  endwhile
  print, 'maxdct=', max(abs(outn))
  FirstT = max(abs(outn))
endif
print, 'FirstT =', FirstT
if not keyword_set(LastT) then LastT = 0
;lambda initialisation
dlambda = (FirstT-LastT)/float(niter)
lambda = FirstT 
;win_image, 0
residual = mca_residual_wl(gm, out_data, mask = mask)
;iterating process of inpainting
for i = 0, niter-1  do begin
  print, 'ITER =', i + 1
  ;lambda evolution (linear or exponential)
  if i gt 0 then begin
    if not keyword_set(expo) then begin
      lambda = lambda - dlambda
    endif else begin
      lambda = LastT +(FirstT-LastT)*(1.-erf(2.8*float(i)/(float(niter))))
    endelse
  endif 	
  if ((lambda lt LastT) or (i eq niter)) then lambda = LastT 
  print, 'lambda =', lambda 
  ;iteration on the number of transform components
  tempo_data = out_data
  if iter eq 0 then tempo_data = tempo_data + residual else tempo_data = residual
  if lambda gt 0 then begin
  im_dct, tempo_data, out , opt='-O -b'+string(DCTBlockSize)
  Threshold = lambda 
  l1 = 0
  l2 = 0
  sz = size(out)
  ndct = sz[1]
  moy = out
  ind = where(abs(out) lt Threshold,c)
  if c gt 0 then out(ind) = 0  
  while l1 lt ndct do begin
    while l2 lt ndct do begin
      ;print, l1,l2
      out(l1,l2) = moy(l1,l2)
      l2 = l2 + DCTBlockSize
    endwhile
    l1 = l1 + DCTBlockSize
    l2 = 0
  endwhile                                                                                  			     
  im_dct, out, tempo_data, opt='-r -O -b'+string(DCTBlockSize)
  if iter eq 1 then out_data = out_data + tempo_data else out_data = tempo_data	 
  if keyword_set(sbounded) then begin
    ;if i lt 0.95*niter then begin
		if (i ne niter-1) then begin
      map  = out_data
      mr_trans, map, wmap, opt='-n'+string(nscale), header  
      ;mr_trans, mm.kappa, win_data, opt='-n'+string(nscale), header 
      for l = 0, nscale-2 do begin
        tempo = wmap(*,*,l)
        ;in_tempo = win_data(*,*,l)
        sigmaint = sigma(tempo(ze))
        sigmaout = sigma(tempo(nze))
        ;sigmaout = sigma(in_tempo(nze))
        if (sigmaint gt 0.2*sigmaout) then begin
				;if (sigmaint gt sigmaout) then begin
          tempo(ze) =  tempo(ze)*(sigmaout/float(sigmaint))
          wmap(*,*,l) = tempo 
        endif
      endfor
      mr_recons, wmap, map, header=header
      out_data = map
    endif	
  endif 
	if keyword_set(sbounded) then begin
    ;if i ge 0.95*niter then begin
		if (i eq niter-1) then begin
      mk_kappa, out_data, n1, n2, field, field, mi
      kappa_to_gamma, mi, gi
					
      mr_trans, gi.gamma1, wgi1, opt='-n'+string(nscale), header  
      mr_trans, gi.gamma2, wgi2, opt='-n'+string(nscale), header  
      ;mr_trans, gm.gamma1, wgm1, opt='-n'+string(nscale), header  
      ;mr_trans, gm.gamma2, wgm2, opt='-n'+string(nscale), header  
				
      for l = 0, nscale-2 do begin
        ;out_tempo1 = wgm1(*,*,l)
        ;out_tempo2 = wgm2(*,*,l)
        int_tempo1 = wgi1(*,*,l)
        int_tempo2 = wgi2(*,*,l)
        sigmaint1 = sigma(int_tempo1(ze))      
        sigmaint2 = sigma(int_tempo2(ze))
        ;sigmaout1 = sigma(out_tempo1(nze))  
        ;sigmaout2 = sigma(out_tempo2(nze))  
				sigmaout1 = sigma(int_tempo1(nze))
				sigmaout2 = sigma(int_tempo2(nze))
        sigint = (sigmaint1 + sigmaint2)/2.
        sigout = (sigmaout1 + sigmaout2)/2.
        ;if ((sigmaint1 gt 0.2*sigmaout1) and (sigmaint2 gt 0.2*sigmaout2)) then begin
        if (sigint gt 0.2*sigout) then begin
				;if (sigint gt sigout) then begin
          ;int_tempo1(ze) =  int_tempo1(ze)*(sigmaout1/float(sigmaint1))
          ;int_tempo2(ze) =  int_tempo2(ze)*(sigmaout2/float(sigmaint2))
          int_tempo1(ze) =  int_tempo1(ze)*(sigout/float(sigint))
          int_tempo2(ze) =  int_tempo2(ze)*(sigout/float(sigint))
          wgi1(*,*,l) = int_tempo1
          wgi2(*,*,l) = int_tempo2
        endif
      endfor
      mr_recons, wgi1, gi.gamma1, header=header
      mr_recons, wgi2, gi.gamma2, header=header
      gamma_to_kappa, gi, mi
      out_data = mi.kappa		 
    endif
		delvarx, mi
		;delvar, mi
		delvarx, gi 
		;delvar, gi
  endif
  if keyword_set(bounded) then begin
    map  = out_data
    ind = where(map gt BoundMax, c)
    if c gt 0 then map(ind) = BoundMax
    ind = where(map lt BoundMin, c)
    if c gt 0 then map(ind) = BoundMin
    out_data = map
  endif			
  residual = mca_residual_wl(gm, out_data, mask = mask)	
  wset, 0
  my_plt, out_data(*,*,0)
  endif
endfor ; endfor i
end 
